@@ -43,13 +43,13 @@ class Dataset(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, DataSourceMixin,
     one. Otherwise, it should be a dictionary matching the format of the *info* mapping. *label* and
     *label_short* are forwarded to the :py:class:`~order.mixins.LabelMixin`, *is_data* to the
     :py:class:`~order.mixins.DataSourceMixin`, *tags* to the :py:class:`~order.mixins.TagMixin`,
-    *aux* to the :py:class:`~order.mixins.AuxDataMixin`, and *name*, *id* and *context* to the
+    *aux* to the :py:class:`~order.mixins.AuxDataMixin`, and *name* and *id* to the
     :py:class:`~order.unique.UniqueObject` constructor.
 
     **Copy behavior**
 
-    All attributes are copied **except** for references to linked processes. The *campaign*
-    reference is kept. Also note the copy behavior of :py:class:`~order.unique.UniqueObject`'s.
+    The :py:attr:`campaign` attribute is carried over as a reference, all remaining attributes are
+    copied. Note that the copied dataset is also registered in the campaign.
 
     **Example**
 
@@ -59,7 +59,9 @@ class Dataset(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, DataSourceMixin,
 
         campaign = od.Campaign("2017B", 1, ...)
 
-        d = od.Dataset("ttH_bb", 1,
+        d = od.Dataset(
+            name="ttH_bb",
+            id=1,
             campaign=campaign,
             keys=["/ttHTobb_M125.../.../..."],
             n_files=123,
@@ -76,7 +78,9 @@ class Dataset(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, DataSourceMixin,
         # -> 123
 
         # similar to above, but set explicit info objects
-        d = Dataset("ttH_bb", 1,
+        d = Dataset(
+            name="ttH_bb",
+            id=1,
             campaign=campaign,
             info={
                 "nominal": {
@@ -140,13 +144,30 @@ class Dataset(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, DataSourceMixin,
     cls_name_plural = "datasets"
 
     # attributes for copying
-    copy_specs = [{"attr": "campaign", "ref": True}, "info"] + UniqueObject.copy_specs + \
-        AuxDataMixin.copy_specs + TagMixin.copy_specs + DataSourceMixin.copy_specs + \
+    copy_specs = (
+        [{"attr": "_campaign", "ref": True}] +
+        UniqueObject.copy_specs +
+        AuxDataMixin.copy_specs +
+        TagMixin.copy_specs +
+        DataSourceMixin.copy_specs +
         LabelMixin.copy_specs
+    )
 
-    def __init__(self, name, id, campaign=None, info=None, processes=None, label=None,
-            label_short=None, is_data=False, tags=None, aux=None, context=None, **kwargs):
-        UniqueObject.__init__(self, name, id, context=context)
+    def __init__(
+        self,
+        name,
+        id,
+        campaign=None,
+        info=None,
+        processes=None,
+        label=None,
+        label_short=None,
+        is_data=False,
+        tags=None,
+        aux=None,
+        **kwargs  # noqa: C816
+    ):
+        UniqueObject.__init__(self, name, id)
         CopyMixin.__init__(self)
         AuxDataMixin.__init__(self, aux=aux)
         TagMixin.__init__(self, tags=tags)
@@ -175,6 +196,15 @@ class Dataset(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, DataSourceMixin,
         """
         return self.get_info(name)
 
+    def copy(self, *args, **kwargs):
+        inst = super(Dataset, self).copy(*args, **kwargs)
+
+        # register in the campaign
+        if inst.campaign:
+            inst.campaign.datasets.add(inst)
+
+        return inst
+
     @property
     def campaign(self):
         # campaign getter
@@ -263,8 +293,7 @@ class DatasetInfo(CopyMixin, AuxDataMixin, TagMixin):
 
     **Copy behavior**
 
-    All attributes are copied. Also note the copy behavior of
-    :py:class:`~order.unique.UniqueObject`'s.
+    All attributes are copied.
 
     **Members**
 
@@ -284,7 +313,10 @@ class DatasetInfo(CopyMixin, AuxDataMixin, TagMixin):
        The number of events.
     """
 
-    copy_specs = ["keys", "n_files", "n_events"] + AuxDataMixin.copy_specs
+    copy_specs = (
+        AuxDataMixin.copy_specs +
+        TagMixin.copy_specs
+    )
 
     def __init__(self, keys=None, n_files=-1, n_events=-1, tags=None, aux=None):
         CopyMixin.__init__(self)
